#include <QtGui>

#include "StrokesArea.h"

double abs(double a) {
	return a < 0? -a : a;
}

StrokesArea::StrokesArea(QWidget* parent) : QWidget(parent) {
	setAttribute(Qt::WA_StaticContents);
	scribbling = false;
	myPenWidth = 10;
	myPenColor = Qt::blue;

	vector.erase(vector.begin(), vector.end());
}

void StrokesArea::clearImage() {
	image.fill(qRgb(255, 255, 255));
	update();
}

void StrokesArea::mousePressEvent(QMouseEvent* event) {
	if(event->button() == Qt::LeftButton) {
		clearImage();
		lastPoint = event->pos();
		vector << event->pos();
		scribbling = true;
	}
}

void StrokesArea::mouseMoveEvent(QMouseEvent* event) {
	if((event->buttons() & Qt::LeftButton) && scribbling) {
		drawLineTo(event->pos());
		vector << event->pos();
	}
}

void StrokesArea::mouseReleaseEvent(QMouseEvent* event) {
	if((event->button() == Qt::LeftButton) && scribbling) {
		int i;
		drawLineTo(event->pos());
		scribbling = false;
		qDebug() << vector.size();
		if(vector.size() < 5) {
			events << (vector.first() + vector.last()) / 2;
			update();
		} else {
			for(i = 0; i < vector.size()-1; i++) {
				double diff = (double)(vector[i+1].y() - vector[i].y())/(double)(vector[i+1].x() - vector[i].x());
				if(abs(diff) < 0.1) {
					events << (vector[i]+vector[i+1])/2;
				}
			}
		}

		QPainter painter(&image);
		painter.setPen(QPen(Qt::red, 20, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
		for(i = 0; i < events.size(); i++) {
			painter.drawPoint(events[i]);
		}
		update();
		painter.setPen(QPen(myPenColor, myPenWidth, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));

		vector.erase(vector.begin(), vector.end());
		events.erase(events.begin(), events.end());
	}
	/* TODO
	 * Here is the critical point. After collecting the positions in a vector,
	 * first of all if they are few or localized, they may be a touch event.
	 * If not, it may be a stroke.
	 * Then a difference vector should be computed (first difference or second
	 * difference? And how much wide?) and used to detect stationary points.
	 * Those points can be used to trigger events.
	 */
}

void StrokesArea::paintEvent(QPaintEvent* /*event*/) {
	QPainter painter(this);
	painter.drawImage(QPoint(0, 0), image);
}

void StrokesArea::resizeEvent(QResizeEvent* event) {
	if(width() > image.width() || height() > image.height()) {
		int newWidth = qMax(width() + 128, image.width());
		int newHeight = qMax(height() + 128, image.height());
		resizeImage(&image, QSize(newWidth, newHeight));
		update();
	}
	QWidget::resizeEvent(event);
}

void StrokesArea::drawLineTo(const QPoint& endPoint) {
	QPainter painter(&image);
	painter.setPen(QPen(myPenColor, myPenWidth, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
	painter.drawLine(lastPoint, endPoint);
	//painter.drawPoint(endPoint);

	int rad = (myPenWidth / 2) + 2;
	update(QRect(lastPoint, endPoint).normalized().adjusted(-rad, -rad, +rad, +rad));
	lastPoint = endPoint;
}

void StrokesArea::resizeImage(QImage* image, const QSize& newSize) {
	if (image->size() == newSize) {
		return;
	}

	QImage newImage(newSize, QImage::Format_RGB32);
	newImage.fill(qRgb(255, 255, 255));
	QPainter painter(&newImage);
	painter.drawImage(QPoint(0, 0), *image);
	*image = newImage;
}
